<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="utf-8">
        <meta name="viewport" content="initial-scale=1.0">
        <title>RQ dashboard</title>

        {# Le styles -#}
        <link rel="shortcut icon" href="{{ url_for('static', filename='favicon.ico') }}">
        <link rel="stylesheet" href="{{ url_for('rq_dashboard.static', filename='css/bootstrap.min.css') }}">
        <link href="{{ url_for('rq_dashboard.static', filename='css/main.css') }}" rel="stylesheet">
    </head>

    <body>
        
        {% include "rq_dashboard/navbar.html" %}
        
        {% if deprecation_options_usage %}
        <p class="alert-danger sticky-top" style="text-align: center;">
            You are using deprecated options, pay attention to --help or readme.md to update your settings!<br>
            {% for option in deprecation_options_usage %}
                option {{ option }} is deprecated<br>
            {% endfor %}
        </p>
        {% endif %}

        <div class="container">
            {% block content %}{% endblock %}
        </div>

        <div id="confirmation-modal" class="modal fade">
            <div class="modal-dialog" role="document">
                <div class="modal-content">
                    <div class="modal-header">
                        <h3>Do you really want to <span id="confirmation-modal-action"></span>?</h3>
                        <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>
                    </div>
                    <div class="modal-footer">
                        <button type="button" id="confirmation-modal-no" class="btn">No</button>
                        <button type="button" id="confirmation-modal-yes" class="btn btn-primary">Yes</button>
                    </div>
                </div>
            </div>
        </div>

        {% include "rq_dashboard/footer.html" %}
        <script src="{{ url_for('rq_dashboard.static', filename='js/jquery-3.4.1.min.js') }}" type="text/javascript"></script>
        <script src="{{ url_for('rq_dashboard.static', filename='js/underscore-umd-min.js') }}" type="text/javascript"></script>
        <script src="{{ url_for('rq_dashboard.static', filename='js/sugar-1.2.1.min.js') }}" type="text/javascript"></script>
        <script src="{{ url_for('rq_dashboard.static', filename='js/bootstrap.bundle.min.js') }}" type="text/javascript"></script>
        <script type="text/javascript">
            {% include "rq_dashboard/inline_js.html" %}
            {% block content_scripts %}{% endblock %}
        </script>


        
    </body>
</html>
