<footer class="footer-text">
    <span class="col-6">
        <a href="https://github.com/Parallels/rq-dashboard">RQ-dashboard</a><span> version: {{ rq_dashboard_version }}
    </span>
    <span class="col-6">
        <a href="http://python-rq.org">Python RQ</a><span> version: {{ rq_version }}
    </span>
</footer>
